//
//  resummationSCET.hh
//  ResHiggs
//
//  Created by Luca Rottoli on 29/10/2014.
//


#pragma once

#include "resummation.hh"

using namespace std;

namespace resumm
{
    
/**
  * Class SCETResummation
  */
class SCETResummation: public Resummation
{
  
private:
  
  bool fPi;

  double   fmus, fmutmuhrat;
  dcomplex fmuh, fmut;
  bool fmusset;

  dcomplex fasmuh, fasmut;
  double   fasmus, fasmuf;
    
  double fGammaA0, fGammaA1, fGammaA2, fGammaA3;
  double fGammaS0, fGammaS1, fGammaS2;
  double fGammaT0, fGammaT1, fGammaT2;
  double fGammaB0, fGammaB1, fGammaB2;
  double fbbeta0, fbbeta1, fbbeta2, fbbeta3;
    
  dcomplex faGA0, faGA1, faGA2, faGA3;
  dcomplex faGS0, faGS1, faGS2, faGS3;
  dcomplex faGB0, faGB1, faGB2, faGB3;
  dcomplex faGT0, faGT1, faGT2, faGT3;
    
  void ComputeCommonSCET(bool);

  void InitAlphas();
  void Eta();
  void H();
  void U();
  dcomplex SoftFunction(dcomplex N);
  dcomplex SoftFunction_LargeN(dcomplex N);
  
  double KFactor();
  
  double StildeFactor(double mus);
  
  double fEta, fEta1, fEta2, fEta3, fEta4;
  double fH0, fH1, fH2, fH3;
  double fULL, fUNLL, fUNNLL, fUNNNLL;
  dcomplex fS0, fS1, fS2, fS3;
  dcomplex fSoft0, fSoft1, fSoft2, fSoft3;

  void InitResummCoeffsSCET();
  void InitCoeffHiggsSCET();
  
  dcomplex FO_1_log_PL(dcomplex N);
  dcomplex FO_2_log_PL(dcomplex N);
  dcomplex FO_3_log_PL(dcomplex N);
  
  dcomplex Subtraction(dcomplex N);
  
  dcomplex StildeOneLoop(dcomplex N);
  dcomplex StildeOneLoop_LargeN(dcomplex N);
  
  dcomplex lum_times_SoftFunction(dcomplex N);
  dcomplex lum_times_Stilde(dcomplex N);
  double SCETMinimalPrescription();
  double StildeMinimalPrescription();
  
  friend double SCETMinimal_integrand(double t, void *p);
  friend double SCETSubtraction_integrand(double t, void *p);
  
  friend double StildeMinimal_integrand(double t, void *p);
  
  
public:
  
  //Set Methods
  void Setmus(double mu) {fmus = mu; fmusset = true; };
  void Resetmus() {fmusset = false; };
  void SetmutmuhRatio(double rat) {fmutmuhrat = rat; };

  //Get Methods
  double GetmusI (double m, double sqrts);
  double GetmusII(double m, double sqrts);
  
  //Pi Resummation
  void UsePi2Resummation(const bool pi=true)  { fPi = pi; };
  
  SCETResummation(process p, collider c, string PDFset, int PDFmember=0): Resummation(p, c, PDFset, PDFmember), fPi(false), fmutmuhrat(1), fmusset(false) { SetPrescription(BP2); SetAPimprovement(AP0); }
  ~SCETResummation();

  // in SCET we use the second scale argument for muH rather than muR
  double res_xs (double m, double sqrt_s, double muF_m_ratio, double muH_m_ratio, log_order LOGo, order match_order, double *asinput=NULL);
  double sigma_contribution(double m, double sqrt_s, double muF_m_ratio, double muH_m_ratio, log_order LOGo, order match_order, double mus, double *asinput=NULL);
  double res_xs_rap(double m, double Y, double sqrt_s, double muF_m_ratio, double muH_m_ratio, log_order LOGo, order match_order, double *asinput=NULL);
  
  };

};
